==> klayout-0.9.tar.gz: klayout-0.9/src/SettingsFormImpl.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <map>

#include <QtGui/QMessageBox>

#include "SettingsFormImpl.h"

#include "layMain.h"
#include "tlLog.h"
#include "dbHershey.h"

// -------------------------------------------------------------

SettingsFormImpl::SettingsFormImpl (QWidget *parent, lay::MainWindow *mw, const char *name, Qt::WFlags fl)
  : QDialog (parent, name, FALSE, fl), Ui::SettingsForm (),
    mp_main_window (mw), m_finalize_recursion (false)
{ 
  Ui::SettingsForm::setupUi (this);

  //  signals and slots connections
  connect( ok_button, SIGNAL( clicked() ), this, SLOT( ok_clicked() ) );
  connect( cancel_button, SIGNAL( clicked() ), this, SLOT( reject() ) );
  connect( apply_button, SIGNAL( clicked() ), this, SLOT( apply_clicked() ) );

  //  Since the designer requires is to create at least one tab, remove this dummy now
  tabWidget->removeTab (0);

  //  Add tabs from the custom scheme for lay::PluginDeclaration 
  std::map <std::string, int> m_tab_headers;
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {

    std::string config_title;
    lay::ConfigPage *config_page = cls->config_page (0, config_title);
    if (config_page) {

      m_config_pages.push_back (config_page);

      std::map <std::string, int>::const_iterator t = m_tab_headers.find (config_title);
      if (t == m_tab_headers.end ()) {

        //  not there yet ..
        QFrame *page = new QFrame (0);
        QVBoxLayout *layout = new QVBoxLayout (page);
        layout->setMargin (0);
        layout->setSpacing (0);
        config_page->setParent (page);
        layout->addWidget (config_page);

        m_tab_headers.insert (std::make_pair (config_title, tabWidget->addTab (page, config_title.c_str ())));

      } else {

        //  already there - add a new item ..
        QFrame *page = dynamic_cast<QFrame *> (tabWidget->widget (t->second));
        if (page) {
          QLayout *layout = page->layout ();
          config_page->setParent (page);
          layout->addWidget (config_page);
        }
   
      }

    }

  }

  for (std::map <std::string, int>::const_iterator t = m_tab_headers.begin (); t != m_tab_headers.end (); ++t) {
    QFrame *page = dynamic_cast<QFrame *> (tabWidget->widget (t->second));
    if (page) {
      QVBoxLayout *layout = dynamic_cast <QVBoxLayout *> (page->layout ());
      if (layout) {
        layout->addStretch (1);
      }
    }
  }

}

void 
SettingsFormImpl::setup ()
{
  //  recursion sentinel
  if (m_finalize_recursion || ! isVisible ()) {
    return;
  }

  //  setup the custom config pages
  for (std::vector <lay::ConfigPage *>::iterator cp = m_config_pages.begin (); cp != m_config_pages.end (); ++cp) {
    (*cp)->setup (mp_main_window);
  }
}

void 
SettingsFormImpl::commit () throw (tl::Exception)
{
  //  commit the custom config pages
  for (std::vector <lay::ConfigPage *>::iterator cp = m_config_pages.begin (); cp != m_config_pages.end (); ++cp) {
    (*cp)->commit (mp_main_window);
  }

  m_finalize_recursion = true;
  try {
    //  config_end will make the main window call setup on the settings form. 
    //  the recursion sentinel takes care of that.
    mp_main_window->config_end ();
    m_finalize_recursion = false;
  } catch (...) {
    m_finalize_recursion = false;
    throw;
  }
}

void 
SettingsFormImpl::ok_clicked ()
{
  try {
    commit ();
    accept ();
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    QMessageBox::critical (this, tl::translate ("Error").c_str (), ex.msg ().c_str ());
  } 
}

void 
SettingsFormImpl::apply_clicked ()
{
  try {
    commit ();
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    QMessageBox::critical (this, tl::translate ("Error").c_str (), ex.msg ().c_str ());
  } 
}

==> klayout-0.10.tar.gz: klayout-0.10/src/SettingsFormImpl.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <map>

#include <QtGui/QMessageBox>

#include "SettingsFormImpl.h"

#include "layMainWindow.h"
#include "tlLog.h"
#include "dbHershey.h"

// -------------------------------------------------------------

SettingsFormImpl::SettingsFormImpl (QWidget *parent, lay::MainWindow *mw, const char *name, Qt::WFlags fl)
  : QDialog (parent, name, FALSE, fl), Ui::SettingsForm (),
    mp_main_window (mw), m_finalize_recursion (false)
{ 
  Ui::SettingsForm::setupUi (this);

  //  signals and slots connections
  connect( ok_button, SIGNAL( clicked() ), this, SLOT( ok_clicked() ) );
  connect( cancel_button, SIGNAL( clicked() ), this, SLOT( reject() ) );
  connect( apply_button, SIGNAL( clicked() ), this, SLOT( apply_clicked() ) );

  //  Since the designer requires is to create at least one tab, remove this dummy now
  tabWidget->removeTab (0);

  //  Add tabs from the custom scheme for lay::PluginDeclaration 
  std::map <std::string, int> m_tab_headers;
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {

    std::string config_title;
    lay::ConfigPage *config_page = cls->config_page (0, config_title);
    if (config_page) {

      m_config_pages.push_back (config_page);

      std::map <std::string, int>::const_iterator t = m_tab_headers.find (config_title);
      if (t == m_tab_headers.end ()) {

        //  not there yet ..
        QFrame *page = new QFrame (0);
        QVBoxLayout *layout = new QVBoxLayout (page);
        layout->setMargin (0);
        layout->setSpacing (0);
        config_page->setParent (page);
        layout->addWidget (config_page);

        m_tab_headers.insert (std::make_pair (config_title, tabWidget->addTab (page, config_title.c_str ())));

      } else {

        //  already there - add a new item ..
        QFrame *page = dynamic_cast<QFrame *> (tabWidget->widget (t->second));
        if (page) {
          QLayout *layout = page->layout ();
          config_page->setParent (page);
          layout->addWidget (config_page);
        }
   
      }

    }

  }

  for (std::map <std::string, int>::const_iterator t = m_tab_headers.begin (); t != m_tab_headers.end (); ++t) {
    QFrame *page = dynamic_cast<QFrame *> (tabWidget->widget (t->second));
    if (page) {
      QVBoxLayout *layout = dynamic_cast <QVBoxLayout *> (page->layout ());
      if (layout) {
        layout->addStretch (1);
      }
    }
  }

}

void 
SettingsFormImpl::setup ()
{
  //  recursion sentinel
  if (m_finalize_recursion || ! isVisible ()) {
    return;
  }

  //  setup the custom config pages
  for (std::vector <lay::ConfigPage *>::iterator cp = m_config_pages.begin (); cp != m_config_pages.end (); ++cp) {
    (*cp)->setup (mp_main_window);
  }
}

void 
SettingsFormImpl::commit () throw (tl::Exception)
{
  //  commit the custom config pages
  for (std::vector <lay::ConfigPage *>::iterator cp = m_config_pages.begin (); cp != m_config_pages.end (); ++cp) {
    (*cp)->commit (mp_main_window);
  }

  m_finalize_recursion = true;
  try {
    //  config_end will make the main window call setup on the settings form. 
    //  the recursion sentinel takes care of that.
    mp_main_window->config_end ();
    m_finalize_recursion = false;
  } catch (...) {
    m_finalize_recursion = false;
    throw;
  }
}

void 
SettingsFormImpl::ok_clicked ()
{
  try {
    commit ();
    accept ();
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    QMessageBox::critical (this, tl::translate ("Error").c_str (), ex.msg ().c_str ());
  } 
}

void 
SettingsFormImpl::apply_clicked ()
{
  try {
    commit ();
  } catch (tl::Exception &ex) {
    tl::error << ex.msg ();
    QMessageBox::critical (this, tl::translate ("Error").c_str (), ex.msg ().c_str ());
  } 
}
